from requests import Request, Session

from hudai import __version__, HudAiError
from hudai.resources import *

USER_AGENT = 'HUD.ai Python v{} +(https://github.com/FoundryAI/hud-ai-python#readme)'.format(__version__)

class HudAi:
    def __init__(self, secret_key, base_url='https://api.hud.ai'):
        if not secret_key:
            raise HudAiError('missing secret_key', 'initialization_error')

        self._secret_key = secret_key
        self._base_url = base_url

        self.article_company = ArticleCompanyResource(self)
        self.article_highlight = ArticleHighlightResource(self)
        self.article_keyterm = ArticleKeytermResource(self)
        self.clean_article = CleanArticleResource(self)
        self.company = CompanyResource(self)
        self.domain = DomainResource(self)
        self.key_term = KeyTermResource(self)
        self.message = MessageResource(self)
        self.news_api_article = NewsApiArticleResource(self)
        self.participant = ParticipantResource(self)
        self.relevant_article = RelevantArticleResource(self)
        self.rss_article = RssArticleResource(self)
        self.rss_feed_metadata = RssFeedMetadataResource(self)
        self.task = TaskResource(self)
        self.text_corpus = TextCorpusResource(self)
        self.user = UserResource(self)

    def make_request(self, method, path, params, data):
        """
        Abstracted request method, request config is defined in the resource itself
        :param method:
        :param path:
        :param params:
        :param data:
        :return:
        """
        session = Session()
        req = Request(method, (self._base_url + path), data=data, params=params)
        prepared = req.prepare()
        prepared.headers['User-Agent'] = USER_AGENT
        prepared.headers['x-api-key'] = self._secret_key
        return session.send(prepared).json()



from hudai.resource import Resource
from pydash import pick


class ArticleHighlightResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client)
        self.resource_name = 'ArticleHighlight'

    def get(self, params):
        return self.request({
            'method': 'GET',
            'params': pick(params, 'url', 'user_id'),
            'url': '/article-highlights/internal/{url}/{user_id}'
        })

    def list(self, params):
        return self.request({
            'method': 'GET',
            'params': pick(params, 'user_id'),
            'url': '/article-highlights/internal'
        })

    def create(self, params):
        return self.request({
            'method': 'POST',
            'data': pick(params, 'url', 'user_id', 'body'),
            'url': '/article-highlights/internal'
        })

    def update(self, params):
        return self.request({
            'method': 'PUT',
            'data': pick(params, 'body'),
            'params': pick(params, 'url', 'user_id'),
            'url': '/article-highlights/internal/{url}/{user_id}'
        })

    def delete(self, params):
        return self.request({
            'method': 'DELETE',
            'params': pick(params, 'url', 'user_id'),
            'url': '/article-highlights/internal/{url}/{user_id}'
        })



from hudai.resource import Resource
from pydash import pick


class KeyTermResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client)
        self.resource_name = 'KeyTerm'


    def associated_users(self, term):
        return self._associated_with(term, 'users')


    def associated_companies(self, term):
        return self._associated_with(term, 'companies')


    def associated_articles(self, term):
        return self._associated_with(term, 'articles')


    def list_for_user(self, user_id):
        return self._list_for('user_id', user_id)


    def list_for_company(self, company_id):
        return self._list_for('company_id', company_id)


    def list_for_article(self, article_id):
        return self._list_for('article_id', article_id)


    def create(self, params):
        return self.request({
            'method': 'POST',
            'data': pick(params, 'keyterm'),
            'url': '/key-terms/internal'
        })


    def get(self, id):
        return self.request({
            'method': 'GET',
            'params': {'id': id},
            'url': '/key-terms/internal/{id}'
        })


    def delete(self, id):
        return self.request({
            'method': 'DELETE',
            'params': {'id': id},
            'url': '/key-terms/internal/{id}'
        })


    def _associated_with(self, term, association):
        return self.request({
            'method': 'GET',
            'params': {'term': term, 'association': association},
            'url': '/key-terms/internal'
        })

    def _list_for(self, key, val):
        return self.request({
            'method': 'GET',
            'params': { key : val },
            'url': '/key-terms/internal'
        })



from hudai.client import HudAi
from hudai.resources import *


def test_initialization():
    client = HudAi('mock-api-key')

    assert isinstance(client, HudAi)
    assert isinstance(client.article_company, ArticleCompanyResource)
    assert isinstance(client.article_highlight, ArticleHighlightResource)
    assert isinstance(client.article_keyterm, ArticleKeytermResource)
    assert isinstance(client.clean_article, CleanArticleResource)
    assert isinstance(client.company, CompanyResource)
    assert isinstance(client.domain, DomainResource)
    assert isinstance(client.message, MessageResource)
    assert isinstance(client.news_api_article, NewsApiArticleResource)
    assert isinstance(client.participant, ParticipantResource)
    assert isinstance(client.relevant_article, RelevantArticleResource)
    assert isinstance(client.rss_article, RssArticleResource)
    assert isinstance(client.rss_feed_metadata, RssFeedMetadataResource)
    assert isinstance(client.task, TaskResource)
    assert isinstance(client.text_corpus, TextCorpusResource)
    assert isinstance(client.user, UserResource)



from hudai.resource import Resource
from pydash import pick


class ParticipantResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client)
        self.resource_name = 'Participant'

    def get(self, id):
        return self.request({
            'method': 'GET',
            'params': {'id': id},
            'url': '/participants/internal/{id}'
        })

    def list(self, params):
        return self.request({
            'method': 'GET',
            'params': pick(params),
            'url': '/participants/internal'
        })

    def create(self, params):
        return self.request({
            'method': 'POST',
            'data': pick(params),
            'url': '/participants/internal'
        })

    def update(self, params):
        return self.request({
            'method': 'PUT',
            'data': pick(params),
            'params': pick(params, 'id'),
            'url': '/participants/internal/{id}'
        })

    def delete(self, id):
        return self.request({
            'method': 'DELETE',
            'params': {'participant': id},
            'url': '/participants/internal/{id}'
        })
